!$Id: constant.f90,v 1.3 2012/04/24 14:50:09 justins Exp $
!-----------------------------------------------------------------
! GEOCAT
!
! Copyright (C) 2006  Michael J. Pavolonis
! National Oceanic and Atmospheric Administration
!
! This program is free software; you can redistribute it and/or
! modify it under the terms of the GNU General Public License
! as published by the Free Software Foundation; either version 2
! of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
! 02110-1301, USA.
!-----------------------------------------------------------------
!-----------------------------------------------------------------
!>For full documentation of this routine, see constant.f90 in the
!!CLAVR-X code/documentation.
!-----------------------------------------------------------------

module CONSTANTS
  implicit none
  integer, parameter, public:: int1 = selected_int_kind(1)
  integer, parameter, public:: int2 = selected_int_kind(3)
  integer, parameter, public:: int4 = selected_int_kind(8)
  integer, parameter, public:: real4 = selected_real_kind(6,37)
  integer, parameter, public:: real8 = selected_real_kind(15,307)
  integer, parameter, public:: ipre = real4
  real (kind=real4), parameter, public:: pi = 3.14159265, dtor = pi/180.0
  real(kind=real8), parameter, public:: missing_value_real8 = -999.0
  real(kind=real4), parameter, public:: missing_value_real4 = -999.0
  integer(kind=int1), parameter, public:: missing_value_int1 = -128
  integer(kind=int2), parameter, public:: missing_value_int2 = -32768
  integer(kind=int4), parameter, public:: missing_value_int4 = -999
  
  real (kind=real4), parameter, public :: c1 = 1.191062e-5, c2 = 1.4387863
  real (kind=real4), parameter, public :: R = 0.28704
  real (kind=real4), parameter, public :: g = 9.8
  real (kind=real4), parameter, public :: solar_constant = 1360.0  ! W/m^2
  
  ! Program Information
  character(*), parameter, public :: EXE_NAME = "geocat"
  character(*), parameter, public :: EXE_PROMPT = "geocat>> "
  character(*), parameter, public :: PVERSION = "1.0"
  character(*), parameter, public :: default_file_name = "geocat.default"
  
  INTEGER(kind=int1), parameter, public :: no_scale = 0, linear_scale = 1, &
                                           log_scale = 2, sqrt_scale = 3

  ! parameters used in retrievals
  real(kind=real4), parameter, public::  cossolzen_min_solar =  0.1    !solar angle limit for daytime


!>Structure containing a variety of symbol constants.  
  TYPE, public ::symbol_struct
    INTEGER(kind=int1) :: CLOUDY = 0
    INTEGER(kind=int1) :: PROB_CLOUDY = 1
    INTEGER(kind=int1) :: PROB_CLEAR = 2
    INTEGER(kind=int1) :: CLEAR = 3
    
    INTEGER(kind=int1) :: CLEAR_TYPE = 0
    INTEGER(kind=int1) :: FOG_TYPE = 1
    INTEGER(kind=int1) :: WATER_TYPE = 2
    INTEGER(kind=int1) :: MIXED_TYPE = 3
    INTEGER(kind=int1) :: TICE_TYPE = 4
    INTEGER(kind=int1) :: CIRRUS_TYPE = 5
    INTEGER(kind=int1) :: OVERLAP_TYPE = 6
    INTEGER(kind=int1) :: UNKNOWN_TYPE = 7
    
    INTEGER(kind=int1) :: CLEAR_PHASE = 0
    INTEGER(kind=int1) :: WATER_PHASE = 1
    INTEGER(kind=int1) :: MIXED_PHASE = 2
    INTEGER(kind=int1) :: ICE_PHASE = 3
    INTEGER(kind=int1) :: UNKNOWN_PHASE = 4
    
    INTEGER(kind=int1) :: NO_AEROSOL_AERO = 0
    INTEGER(kind=int1) :: MOSTLY_ASH_AERO = 1
    INTEGER(kind=int1) :: ASH_ICE_AERO = 2
    INTEGER(kind=int1) :: DUST_AERO = 3
    INTEGER(kind=int1) :: SMOKE_AERO = 4
    INTEGER(kind=int1) :: SULFATE_AERO = 5
    INTEGER(kind=int1) :: UNKNOWN_AERO = 6
    
    INTEGER(kind=int1) :: NO_SPACE = 0
    INTEGER(kind=int1) :: SPACE =1 
    
  END TYPE symbol_struct
  
!>Make symbol_struct available to other routines in snow/ice code
  TYPE(symbol_struct), public, SAVE :: sym
  
!>Timing structure
  TYPE, public :: time_struct
    integer (kind=int4) :: ileap
    integer (kind=int4) :: year
    integer (kind=int4) :: month
    integer (kind=int4) :: jday
    integer (kind=int4) :: day
    integer (kind=int4) :: hour
    integer (kind=int4) :: minute
    integer (kind=int4) :: second
    real (kind=real4) :: hour_frac
  END TYPE time_struct
  
  CONTAINS

!------------------------------------------------------------------------
!>This routine computes time elapsed between counter start and stop.
!!                                  
!! AUTHOR:\n                        
!!   Mike Pavolonis (NOAA/NESDIS)   
!!                                  
!! DATE:\n
!!   04/17/12
!!
!! PARAMETERS:\n
!!   clock_count_start - Clock counter start value\n
!!   clock_count_end - Clock counter end value\n
!!   clock_rate - Clock rate value\n
!!   option - Optional input to specify output format\n
!!   Function Result - Prints processing time to screen and 
!!                     returns elapsed time.
!------------------------------------------------------------------------
  
  FUNCTION time_elapsed(clock_count_start, &
                        clock_count_end, &
		        clock_rate, &
			option) result(tdiff)
			      
    INTEGER, intent(in) :: clock_count_start, clock_count_end, clock_rate
    INTEGER, intent(in) :: option
    REAL (kind=real4) :: tdiff
    INTEGER :: dd, hh, mm, ss
    
    tdiff = (clock_count_end-clock_count_start)/real(clock_rate,kind=real4)
    dd = int(tdiff/86400)
    hh = int((int(modulo(tdiff,86400.0)))/3600)
    mm = int((int(modulo(tdiff,3600.0)))/60)
    ss = int(int(modulo(tdiff,60.0)))
    
    if (option == 1) then
      print "(a,'Total processing time:', &
            'dd=',i0,', hh=',i0,', mm=',i0,', ss=',i0)", &
	    EXE_PROMPT,dd,hh,mm,ss
    else if (option == 2) then
      print "(/,a,'Total processing time:',/, &
            '     Days = 'i0,/, &
	    '     Hours = 'i0,/, &
	    '     Minutes = 'i0,/, &
	    '     Seconds = 'i0,/)",EXE_PROMPT,dd,hh,mm,ss
    endif
    
END FUNCTION time_elapsed

end module CONSTANTS
