! $Id: julian_utils_mod.f90,v 1.2 2012/04/24 14:50:09 justins Exp $
!------------------------------------------------------------------
!>The set of routines in this module is used to convert a 
!!3-digit julian day to its corresponding month and day of month
!!or vice verse. Leap year is taken into account.
!------------------------------------------------------------------
module JULIAN_UTILS

implicit none

private
public::julian2yrmthdy, yrmthdy2julian

contains

!------------------------------------------------------------------------
!>This routine converts julian date (YYYY-JJJ) to regular date 
!!(YYYY-MM-DD).
!!                                  
!! AUTHOR:\n                        
!!   Mike Pavolonis (NOAA/NESDIS)   
!!                                  
!! DATE:\n
!!   04/17/12
!!
!! PARAMETERS:\n
!!   year- Input year to convert\n
!!   jdy - Input julian day to convert\n
!!   mth - Output caldenar month\n
!!   dy - Output day of month
!------------------------------------------------------------------------

SUBROUTINE julian2yrmthdy(year,jdy,mth,dy)

implicit none

integer,intent(in) :: year,jdy
integer,intent(out) :: mth,dy
integer :: i,lpyrck
integer,dimension(12) :: jday

!print *, 'Date entered:'
!print *, year,jdy

lpyrck = mod(float(year),4.0)
!print *, lpyrck

if (lpyrck == 0) then
! print *, 'year is a leap year'
 jday(1) = 1
 jday(2) = 32
 jday(3) = 61
 jday(4) = 92
 jday(5) = 122
 jday(6) = 153
 jday(7) = 183
 jday(8) = 214
 jday(9) = 245
 jday(10) = 275
 jday(11) = 306
 jday(12) = 336

 do i=1,12
  mth = i
  if (jdy < jday(i)) then
   mth = i-1
   exit
  endif
 enddo
 dy = jdy-jday(mth)+1
! print *, 'returned year, month, day:'
! print *, year,mth,dy
endif

if (lpyrck /= 0) then
! print *, 'year is not a leap year'
 jday(1) = 1
 jday(2) = 32
 jday(3) = 60
 jday(4) = 91
 jday(5) = 121
 jday(6) = 152
 jday(7) = 182
 jday(8) = 213
 jday(9) = 244
 jday(10) = 274
 jday(11) = 305
 jday(12) = 335

 do i=1,12
  mth = i
  if (jdy < jday(i)) then
   mth = i-1
   exit
  endif
 enddo
 dy = jdy-jday(mth)+1
! print *, 'returned year, month, day:'
! print *, year,mth,dy
endif

end subroutine julian2yrmthdy
!********************************************************************

!------------------------------------------------------------------------
!>This routine converts regualr date (YYYY-MM-DD) to julian (YYYY-JJJ).
!!                                  
!! AUTHOR:\n                        
!!   Mike Pavolonis (NOAA/NESDIS)   
!!                                  
!! DATE:\n
!!   04/17/12
!!
!! PARAMETERS:\n
!!   year- Input year to convert\n
!!   mth - Input caldenar month to convert\n
!!   dy - Input day of month to convert\n
!!   jdy - Output day of year (julian date)
!------------------------------------------------------------------------
SUBROUTINE yrmthdy2julian(year,mth,dy,jdy)

implicit none

integer,intent(in) :: year,mth,dy
integer,intent(out) :: jdy
integer :: i,lpyrck
integer,dimension(12) :: jday

!print *, 'Date entered:'
!print *, year,mth,dy

lpyrck = mod(float(year),4.0)
!print *, lpyrck

if (lpyrck == 0) then
! print *, 'year is a leap year'
 jday(1) = 1
 jday(2) = 32
 jday(3) = 61
 jday(4) = 92
 jday(5) = 122
 jday(6) = 153
 jday(7) = 183
 jday(8) = 214
 jday(9) = 245
 jday(10) = 275
 jday(11) = 306
 jday(12) = 336

 jdy = jday(mth)+dy-1

! print *, 'returned year, jday:'
! print *, year,jdy

endif

if (lpyrck /= 0) then
! print *, 'year is not a leap year'
 jday(1) = 1
 jday(2) = 32
 jday(3) = 60
 jday(4) = 91
 jday(5) = 121
 jday(6) = 152
 jday(7) = 182
 jday(8) = 213
 jday(9) = 244
 jday(10) = 274
 jday(11) = 305
 jday(12) = 335

 jdy = jday(mth)+dy-1

! print *, 'returned year, jday:'
! print *, year,jdy

endif

end subroutine yrmthdy2julian

end module JULIAN_UTILS
